import uuid
from dataclasses import field
from typing import Generic, TypeVar

from typing_extensions import Self

from pydantic import model_serializer, BaseModel

T = TypeVar('T')


class Id(BaseModel, Generic[T]):
	value: uuid.UUID

	def __init__(self, value: uuid.UUID = None):
		value = value if value is not None else uuid.uuid4()
		super().__init__(value=value)
		self.value = value

	@staticmethod
	def field():
		return field(default_factory=Id)

	def __str__(self):
		return str(self.value)

	def __eq__(self, other: Self):
		return self.value == other.value

	def __hash__(self):
		return hash(self.value)

	@model_serializer
	def model_serializer(self) -> str:
		return self.__str__()
